// All category items MUST use the following prefix.
var catPrefix = '---';

function setupEditableComboBox(id, eventHandler) {
	$.widget("custom.combobox", {
		_create: function() {
			this.wrapper = $("<span>").addClass("custom-combobox").insertAfter(this.element);
			this.element.hide();
			this._createAutocomplete();

			var obj = $('#' + id);
			obj[0].eventHandler = eventHandler;
		},

		_createAutocomplete: function() {
			var selected = this.element.children(":selected"),
			value = selected.val() ? selected.text() : "";
			
			$('#' + id + '_InpContainer').appendTo(this.wrapper);
			$('#' + id + '_Btn').click(function() {
				if ($('#' + id + '_Inp').hasClass('keep-focus')==false) {
					$('#' + id + '_Inp').autocomplete('search', '');
				}
			})

			this.input = $('#' + id + '_Inp')
				.click(function() {
					if ($(this).val()=='') {
						$(this).autocomplete("search", $(this).val());
					}
				})
				.val(value)
				.attr('title', '')
				.autocomplete({
					delay: 0,
					minLength: 0,
					source: this._source.bind( this ),
	
					close: function (event, ui) {
						var id = $(this).attr('id');
						setTimeout("$('#" + id + "').removeClass('keep-focus');", 250);
					},
					
					open: function (event, ui) {
						$(this).addClass('keep-focus');

			            var menu = $(this).data("uiAutocomplete").menu;
			            var items = $('li', menu.element);
			            
			            // Mark category lines.
			            for (var i = 0; i < items.length; i++) {
							var txt = $(items[i]).html();

							// Align part of the text to the right.
							if (txt.indexOf(' ~ ')>0) {
								txt = txt.replace(' ~ ', '<span class="float-right">');
								txt = txt.replace('</div>', '</span></div>');
								$(items[i]).html(txt);
							}

							if (txt.indexOf(catPrefix)>=0) {
								$(items[i]).attr('class', 'ps-autocomp-category');
							}
						}

			            if ($(this).val().length > 0) {
			                for (var i = 0; i < items.length; i++) {
			                    if (items.eq(i).text().toUpperCase().includes($(this).val().toUpperCase())) {
			                        menu.focus(null, items.eq(i));
			                        break;
			                    }
			                }
			            }
					}
				});

			this._on( this.input, {
				autocompleteselect: function( event, ui ) {
					ui.item.option.selected = true;

					var obj = $('#' + id);
					var eh = obj[0].eventHandler;
					if (eh!=null) {
						var li = $(ui.item.option);
						eh(li.val(), li.html(), 'SELECTED');
					}

					this._trigger("select", event, {
						item: ui.item.option
					});
				},

				autocompletechange: "_removeIfInvalid"
			});
		},

		_source: function(request,response) {
			var matcher = new RegExp('\\b' + $.ui.autocomplete.escapeRegex(request.term), "i");

			response(this.element.children("option").map(function() {
				var text = $( this ).text();

				if (this.value && (!request.term || matcher.test(text) || text.indexOf(catPrefix)>=0)) {
					return {
						label: text,
						value: text,
						option: this
					};
				}
			}) );
		},

		_removeIfInvalid: function(event, ui) {
			// Selected an item, nothing to do
			if (ui.item) {
				return;
			}

			// Search for a match (case-insensitive)
			var value = this.input.val(),
			valueLowerCase = value.toLowerCase(),
			valid = false;

			this.element.children( "option" ).each(function() {
				if ( $( this ).text().toLowerCase() === valueLowerCase ) {
					this.selected = valid = true;
					return false;
				}
			});

			// Found a match, nothing to do
			if (valid) {
				return;
			}

			var obj = $('#' + id);
			var eh = obj[0].eventHandler;
			if (eh!=null) {
				var txt = this.input.val();
				eh(txt, txt, 'NOT-FOUND');
			}

			// Remove invalid value
			this.input.val('');
			this.element.val('');
			this.input.autocomplete('instance').term = '';
		},

		_destroy: function() {
			this.wrapper.remove();
			this.element.show();
		}
	});

	$('#' + id).combobox();
};
